var metaObj = {
    "TITLE_ID": {
        "value" : null,
        "bytes" : null,
        "bytesLen" : 9
    },
    "PS3_SYSTEM_VER": {
        "value" : null,
        "bytes" : null,
        "bytesLen" : 4
    },
    "VERSION": {
        "value" : null,
        "bytes" : null,
        "bytesLen" : 5
    },
    "APP_VER": {
        "value" : null,
        "bytes" : null,
        "bytesLen" : 5
    }
};

var indexOffset;

var showAlert = true;

function GlbVarInit(){
    metaObj["TITLE_ID"].value = null;
    metaObj["TITLE_ID"].bytes = null;
    metaObj["PS3_SYSTEM_VER"].value = null;
    metaObj["PS3_SYSTEM_VER"].bytes = null;
    metaObj["VERSION"].value = null;
    metaObj["VERSION"].bytes = null;
    metaObj["APP_VER"].value = null;
    metaObj["APP_VER"].bytes = null;
    indexOffset = 0;
    showAlert = true;
}

// ファイルの読み込み
function loadFiles(file){
    var reader = new FileReader();
    reader.onload = function(){
        handleArrayBuffer(reader.result);
    }
    reader.readAsArrayBuffer(file);
}

// ArrayBufferを処理
function handleArrayBuffer(sfoArrayBuffer){
    GlbVarInit();
    (function(){
        var def = $.Deferred();
        parseSfo(sfoArrayBuffer, def)
        return def.promise();
    }()).then(
        function(crc32){
            var def = $.Deferred();
            getIrdName(crc32, def);
            return def.promise();
        },
        function(str){
            if(showAlert){
                alert(str);
                showAlert = false;
            }
            return $.Deferred().reject().promise();
        }
    ).then(
        function(data){
            var def = $.Deferred();
            outputLink(data, def);
            return def.promise();
        },
        function(){
            if(showAlert){
                alert("ird not found.");
                showAlert = false;
            }
            return $.Deferred().reject().promise();
        }
    ).then(
        function(){
            //
        },
        function(){
            if(showAlert){
                alert("ird not found.");
                showAlert = false;
            }
        }
    )
}

function parseSfo(sfoArrayBuffer, def){
    if(sfoArrayBuffer.byteLength >= 20){
        var u8Array = new Uint8Array(sfoArrayBuffer);
        var sfoHeader = {
            "magic":readUint32(u8Array),
            "version":readUint32(u8Array),
            "keyofs":readUint32(u8Array),
            "valofs":readUint32(u8Array),
            "count":readUint32(u8Array)
        }

        if(sfoHeader.magic == "0x46535000"){
            var sfoIndexes = [];
            for(var i=0; i<sfoHeader.count; i++){
				var sfoIndex = {
					"nameofs":readUint16(u8Array),
					"alignment":readUint8(u8Array),
					"type":readUint8(u8Array),
					"valsize":readUint32(u8Array),
					"totalsize":readUint32(u8Array),
					"dataofs":readUint32(u8Array)
				}
				sfoIndexes = sfoIndexes.concat(sfoIndex);
			}

            var searchKeyArray = ["TITLE_ID", "PS3_SYSTEM_VER", "VERSION", "APP_VER"];
            try{
                for(var i=0; i<sfoIndexes.length; i++){
                    var keyOffset = sfoHeader.keyofs + sfoIndexes[i].nameofs;
                    var keySize = keyOffset + readKeyBytes(u8Array, keyOffset);
                    var keyName = readString(u8Array, keyOffset, keySize);
                    var valOffset = sfoHeader.valofs + sfoIndexes[i].dataofs;
                    var valSize = valOffset + sfoIndexes[i].valsize - 1;
                    if(searchKeyArray.indexOf(keyName)>-1){
                        var value = readString(u8Array, valOffset, valSize);
                        if(keyName=="PS3_SYSTEM_VER"){
                            value = value.slice(1,5);
                        }
                        var bytes = stringToBytes(value);
                        metaObj[keyName].value = value;
                        metaObj[keyName].bytes = bytes;
                    }
                }
                console.log(metaObj);
                
                var data = []
                $.each(searchKeyArray, function(i, keyName){
                    var bytes = metaObj[keyName].bytes;
                    if(bytes==null){
                        bytes = [];
                        for(var j=0; j<metaObj[keyName].bytesLen; j++){
                            bytes.push(0);
                        }
                    }
                    data = data.concat(bytes);
                })

                const crc32 = (CRC32.buf(data) >>> 0).toString(16).toUpperCase();
                def.resolve(crc32);
            }catch(e){
                def.reject(e)
            }
        }else{
            def.reject("This file is not SFO.");
        }
    }else{
        def.reject("This file is less than 20 bytes. Cannot get the header.")
    }
}


function readUint8(u8Array){
    var u8 = u8Array[indexOffset];
    indexOffset++;
    return u8;
}

function readUint16(u8Array){
    var ary = u8Array.slice(indexOffset, indexOffset+2);
    var u16 = new Uint16Array(ary.buffer)[0];
    indexOffset += 2;
    return u16;
}

function readUint32(u8Array){
    var ary = u8Array.slice(indexOffset, indexOffset+4);
    var u32 = new Uint32Array(ary.buffer)[0];
    indexOffset += 4;
    return u32;
}

function readKeyBytes(u8Array, keyOffset){
    for(var keySize=0; ; keySize++){
        u8 = u8Array[keyOffset+keySize];
        if(u8 == 0x00){
            return keySize;
        }
    }
}

function readString(u8Array, strOffset, strSize){
    var ary = u8Array.slice(strOffset, strSize);
    return new TextDecoder("utf-8").decode(ary);
}

function readHex(u8Array, hexOffset, hexSize){
    var ary = u8Array.slice(hexOffset, hexSize);
    var hexAry = [];
    for(var i=0; i<ary.length; i++){
        var hex = ary[i].toString(16);
        if(hex.length==1){
            hex = "0" + hex;
        }
        hexAry.push(hex);
    }
    return hexAry.join("");
}

function manualMode(titleID, ps3SystemVer, version, appVer){
    GlbVarInit();
    metaObj["TITLE_ID"].value = titleID;
    metaObj["PS3_SYSTEM_VER"].value = ps3SystemVer;
    metaObj["VERSION"].value = version;
    metaObj["APP_VER"].value = appVer;
    var keyNameArray = ["TITLE_ID", "PS3_SYSTEM_VER", "VERSION", "APP_VER"];
    var data = [];
    $.each(keyNameArray, function(i, keyName){
        var value = metaObj[keyName].value;
        var bytes;
        if(value==""){
            bytes = [];
            for(var j=0; j<metaObj[keyName].bytesLen; j++){
                bytes.push(0);
            }
        }else{
            bytes = stringToBytes(value);
        }
        metaObj[keyName].bytes = bytes;
        data = data.concat(bytes);
    })
    console.log(metaObj);

    const crc32 = (CRC32.buf(data) >>> 0).toString(16).toUpperCase();

    (function(){
        var def = $.Deferred();
        getIrdName(crc32, def);
        return def.promise();
    }()).then(
        function(data){
            var def = $.Deferred();
            outputLink(data, def);
            return def.promise();
        },
        function(){
            if(showAlert){
                alert("ird not found.");
                showAlert = false;
            }
            return $.Deferred().reject().promise();
        }
    ).then(
        function(){
            //
        },
        function(){
            if(showAlert){
                alert("ird not found.");
                showAlert = false;
            }
        }
    )
}

function getIrdName(crc32, def){
    var ps3irdURL = "http://ps3ird.free.fr/script.php?ird=" + crc32;
    ps3irdURL = encodeURIComponent(ps3irdURL);
    $.ajax({
		type: "GET",
		url: "scraper.php?url=" + ps3irdURL
	}).done(function(data){
        console.log(data);
        if(data.indexOf(".ird")>-1){
            def.resolve(data);
        }else{
            def.reject();
        }
	}).fail(function(){
		def.reject();
	});	
}

function outputLink(data, def){
    $("#mode-select-wrapper").hide();
    $("#drop-area-wrapper").hide();
    $("#manual-input-area-wrapper").hide();
    $("#output-area-wrapper").show();
    const irdLen = data.match(/\.ird/g).length;
    if(irdLen!=0){
        var irdNameArray = data.split("\n");
        if(irdNameArray.length != irdLen){
            irdNameArray = data.split("\r");
        }
        const linkElemArray = [];
        $.each(irdNameArray, function(i, ird){
            const irdURL = "http://ps3ird.free.fr/" + ird;
            const linkElem = "<a href='"+irdURL+"'>" + ird + "</a>";
            linkElemArray.push(linkElem);
        })
        const irdElem = linkElemArray.join("<br>");
        const out =  $("#output-area");
        outputItem(out, "TITLE_ID", metaObj["TITLE_ID"].value);
        outputItem(out, "PS3_SYSTEM_VER", metaObj["PS3_SYSTEM_VER"].value);
        outputItem(out, "VERSION", metaObj["VERSION"].value);
        outputItem(out, "APP_VER", metaObj["APP_VER"].value);
        out.append("<br>");
        out.append(irdElem);
        def.resolve();
    }else{
        def.reject();
    }
}

function outputItem(out, itemName, str){
    var item = "<span class='item_str'>" + str + "</span>";
    item = "<span class='item_name'>" + itemName + " : </span>" + item;
    item = "<span class='item'>" + item + "</span>";
    item += "<br>";
    out.append(item);
}

function stringToBytes(str){
    var bytes = str.split("").map(function(char){
        return char.charCodeAt(0);
    })
    return bytes;
}
